<?php

function docebo_lms_general_settings_form() {
	$form = array();

	$form['scenario'] = array(
		'#type' => 'radios',
		'#title' => t('Choose your users\' scenario'),
		'#options' => array(
			'login_mask' => t('I already have users in Docebo and I want to keep them separated from Drupal users'),
			'sso' => t('I already have users in Drupal and I want to create them in Docebo, so they can log in with the same username and password (Single Sign-on System)'),
		),
		'#default_value' => variable_get('docebo_lms_scenario', 'login_mask'),
		'#parents' => array('params', 'scenario'),
		'#required' => true,
	);

	$form['separator'] = array(
		'#markup' => '<hr/>',
	);

	$form['cloud_address'] = array(
		'#type' => 'textfield',
		'#title' => t('Docebo Cloud Address'),
		'#parents' => array('params', 'cloud_address'),
		'#default_value' => variable_get('docebo_lms_cloud_address', ''),
	);

	$form['key'] = array(
		'#type' => 'textfield',
		'#title' => t('Key'),
		'#parents' => array('params', 'key'),
		'#default_value' => variable_get('docebo_lms_key', ''),
	);

	$form['secret_key'] = array(
		'#type' => 'textfield',
		'#title' => t('Secret Key'),
		'#parents' => array('params', 'secret_key'),
		'#default_value' => variable_get('docebo_lms_secret_key', ''),
	);

	$form['sso'] = array(
		'#type' => 'textfield',
		'#title' => t('SSO'),
		'#parents' => array('params', 'sso'),
		'#default_value' => variable_get('docebo_lms_sso', ''),
	);

	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Save changes'),
	);

	$form['#attached'] = array(
		'css' => array(
			'type' => 'file',
			'data' => drupal_get_path('module', 'docebo_lms') . '/css/docebo.forms.css',
		),
	);

	$api = new DoceboApi();
	$connection = $api->checkConnection();

	$class = (isset($connection->success) && $connection->success == 1) ? 'success' : 'failure';
	$status = isset($connection->api_status) ? t(ucfirst($connection->api_status)) : t('Not available');

	$statusMarkup = "<h4 class='connection-status $class'>" . t('API Connection') .": <span>$status</span></h4>";

	$form['api_status'] = array(
		'#markup' => $statusMarkup,
	);

	return $form;
}

function docebo_lms_general_settings_form_submit(&$form, &$form_state) {
	$form_state['values']['params']['cloud_address'] = preg_replace('/^https?:\/\//', '', trim($form_state['values']['params']['cloud_address']));

	foreach ($form_state['values']['params'] as $key => $value) {
		variable_set('docebo_lms_' . $key, trim($value));
	}

	drupal_set_message(t('Configuration options have been saved'));
}

function docebo_lms_login_settings_form($form, &$form_state) {
	$scenario = variable_get('docebo_lms_scenario', 'login_mask');

	if ($scenario == 'sso') {
		if (!isset($_REQUEST['ajax_iframe_upload'])) {
			drupal_set_message(t('!url is also available - your users may use that to log in to your LMS.', array('!url' => l(t('Single Sign-On'), 'admin/config/services/docebo/sso'))), 'status', false);
		}
	}

	$key = 'docebo_lms_login_';

	$form['#attached'] = array(
		'library' => array(
			array('system', 'farbtastic'),
		),
		'js' => array(drupal_get_path('module', 'docebo_lms') . '/js/script.js'),
	);

	$form['introduction'] = array(
		'#markup' => '<p class="introduction">' . t('Use this box if you want to allow your users to log in to Docebo LMS directly from your Drupal website BUT not with the single sing-on system.') . '</p>',
	);

	$form['logo'] = array(
		'#type' => 'managed_file',
		'#title' => t('Login box logo'),
		'#description' => t('Remove this image to use Docebo logo'),
		'#default_value' => variable_get($key . 'logo', null),
		'#progress_indicator' => 'bar',
		'#upload_location' => 'public://docebo/',
		'#upload_validators' => array('file_validate_is_image' => array()),
		'#parents' => array('params', 'logo'),
	);

	$form['do_not_use_logo'] = array(
		'#type' => 'checkbox',
		'#title' => t('Do not use logo'),
		'#parents' => array('params', 'do_not_use_logo'),
		'#default_value' => variable_get($key . 'do_not_use_logo', 0),
	);

	$form['intro_text'] = array(
		'#type' => 'textarea',
		'#title' => t('Intro text'),
		'#resizable' => false,
		'#rows' => 5,
		'#parents' => array('params', 'intro_text'),
		'#default_value' => variable_get($key . 'intro_text', ''),
	);

	$form['box_width'] = array(
		'#type' => 'textfield',
		'#title' => t('Box width'),
		'#field_suffix' => t('px'),
		'#parents' => array('params', 'box_width'),
		'#default_value' => variable_get($key . 'box_width', ''),
	);

	$form['box_height'] = array(
		'#type' => 'textfield',
		'#title' => t('Box height'),
		'#field_suffix' => t('px'),
		'#parents' => array('params', 'box_height'),
		'#default_value' => variable_get($key . 'box_height', ''),
	);

	$form['login_btn_text'] = array(
		'#type' => 'textfield',
		'#title' => t('Login button text'),
		'#parents' => array('params', 'login_btn_text'),
		'#default_value' => variable_get($key . 'login_btn_text', ''),
	);

	$form['login_btn_position'] = array(
		'#type' => 'radios',
		'#title' => t('Button position'),
		'#parents' => array('params', 'login_btn_position'),
		'#default_value' => variable_get($key . 'login_btn_position', 'left'),
		'#options' => array(
			'left' => t('Left'),
			'center' => t('Center'),
			'right' => t('Right'),
		),
		'#required' => true,
	);

	$form['login_box_intro'] = array(
		'#markup' => '<h4>' . t('Customize your Login box') . '</h4>', 
	);

	$customCss = variable_get($key . 'custom_css', '');

	if ($customCss == '') {
		$form['edit_css'] = array(
			'#markup' => '<a href="javascript:void(0);" class="custom-css-toggle">' . t('Manually edit the CSS') . '</a>',
		);	
	}

	$form['custom_css'] = array(
		'#type' => 'textarea',
		'#title' => t('Custom CSS'),
		'#resizable' => false,
		'#rows' => 5,
		'#attributes' => array(
			'class' => array('custom-css'),
		),
		'#parents' => array('params', 'custom_css'),
		'#default_value' => variable_get($key . 'custom_css', ''),
	);

	$form['text_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Text color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'text_color'),
		'#default_value' => variable_get($key . 'text_color', ''),
	);

	$form['border_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Border color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'border_color'),
		'#default_value' => variable_get($key . 'border_color', ''),
	);

	$form['background_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Background color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'background_color'),
		'#default_value' => variable_get($key . 'background_color', ''),
	);

	$form['use_box_shadow'] = array(
		'#type' => 'checkbox',
		'#title' => t('Use box shadow?'),
		'#parents' => array('params', 'use_box_shadow'),
		'#default_value' => variable_get($key . 'use_box_shadow', 0),
	);

	$form['btn_background_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button background color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_background_color'),
		'#default_value' => variable_get($key . 'btn_background_color', ''),
	);

	$form['btn_border_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button border color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_border_color'),
		'#default_value' => variable_get($key . 'btn_border_color', ''),
	);

	$form['btn_text_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button text color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_text_color'),
		'#default_value' => variable_get($key . 'btn_text_color', ''),
	);

	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => 'Save',
	);

	$previewForm = drupal_get_form('docebo_lms_login_form', true);
	$form['#suffix'] = '<div id="preview-wrapper" class="login"><h4>' . t('Preview') . '</h4><div id="preview">'.drupal_render($previewForm).'</div></div>';

	return $form;
}

function docebo_lms_login_settings_form_validate($form, &$form_state) {
	$values = $form_state['values']['params'];

	if (!empty($values['box_width'])) {
		if (!is_numeric($values['box_width'])) {
			form_set_error('params][box_width', 'Box width must be a numeric value');
		} elseif ($values['box_width'] <= 0) {
			form_set_error('params][box_width', 'Box width must be greater than zero');
		}
	}

	if (!empty($values['box_height'])) {
		if (!is_numeric($values['box_height'])) {
			form_set_error('params][box_height', 'Box height must be a numeric value');
		} elseif ($values['box_height'] <= 0) {
			form_set_error('params][box_height', 'Box height must be greater than zero');
		}
	}
}

function docebo_lms_login_settings_form_submit($form, &$form_state) {
	global $user;

	$fid = $form_state['values']['params']['logo'];

	if ($fid) {
		$file = file_load($fid);
		if ($file->status == 0) {
			$file->status = FILE_STATUS_PERMANENT;
			file_save($file);
		}
		file_usage_add($file, 'docebo_lms', 'user', 0);
	} else {
		$fid = variable_get('docebo_lms_login_logo', null);
		if ($fid) {
			$file = file_load($fid);
			file_usage_delete($file, 'docebo_lms', 'user', 0);
		}

		variable_set('docebo_lms_login_logo', null);
	}

	foreach ($form_state['values']['params'] as $key => $value) {
		variable_set('docebo_lms_login_' . $key, $value);
	}

	$settings = docebo_lms_load_settings('docebo_lms_login_');

	docebo_lms_generate_css('docebo.login', $settings);
	docebo_lms_generate_custom_css('docebo.login.custom', $settings);
	_drupal_flush_css_js();

	drupal_set_message(t('Configuration options have been saved'));
}

function docebo_lms_sso_settings_form($form, &$form_state) {
	$key = 'docebo_lms_sso_';

	$scenario = variable_get('docebo_lms_scenario', 'login_mask');

	if ($scenario == 'login_mask') {
		drupal_set_message(t('Single Sign-on is not enabled - please use the !url instead.', array('!url' => l(t('Login Box'), 'admin/config/services/docebo/login'))), 'error');
		return array();
	}

	$form['#attached'] = array(
		'library' => array(
			array('system', 'farbtastic'),
		),
		'js' => array(drupal_get_path('module', 'docebo_lms') . '/js/script.js'),
	);

	// Form preview
	$previewForm = drupal_get_form('docebo_lms_sso_form', true);
	$form['preview'] = array(
		'#markup' => '<div id="preview-wrapper" class="login"><h4>' . t('Preview') . '</h4><div id="preview">'.drupal_render($previewForm).'</div></div>',
	);

	$form['introduction'] = array(
		'#markup' => '<p class="introduction">' . t('Use this box if you already use a login system into your Drupal website and you want to allow your users to log in to Docebo LMS with the Single Sing-On system.') . '</p>',
	);

	$form['introduction_1'] = array(
		'#markup' => '<h4>' . t('SSO box') . '</h4>',
		'#description' => t('You can also display a box with your latest progress and available courses from Docebo Cloud')
	);

	$form['box_title'] = array(
		'#type' => 'textfield',
		'#title' => t('Box title'),
		'#parents' => array('params', 'box_title'),
		'#default_value' => variable_get($key . 'box_title', ''),
	);

	$form['intro_text'] = array(
		'#type' => 'textarea',
		'#title' => t('Intro text'),
		'#resizable' => false,
		'#rows' => 5,
		'#parents' => array('params', 'intro_text'),
		'#default_value' => variable_get($key . 'intro_text', ''),
	);

	$form['box_width'] = array(
		'#type' => 'textfield',
		'#title' => t('Box width'),
		'#field_suffix' => t('px'),
		'#parents' => array('params', 'box_width'),
		'#default_value' => variable_get($key . 'box_width', ''),
	);

	$form['box_height'] = array(
		'#type' => 'textfield',
		'#title' => t('Box height'),
		'#field_suffix' => t('px'),
		'#parents' => array('params', 'box_height'),
		'#default_value' => variable_get($key . 'box_height', ''),
	);

	$form['btn_text'] = array(
		'#type' => 'textfield',
		'#title' => t('SSO button text'),
		'#parents' => array('params', 'btn_text'),
		'#default_value' => variable_get($key . 'btn_text', ''),
	);

	$form['btn_position'] = array(
		'#type' => 'radios',
		'#title' => t('Button position'),
		'#parents' => array('params', 'btn_position'),
		'#default_value' => variable_get($key . 'btn_position', 'left'),
		'#options' => array(
			'left' => t('Left'),
			'center' => t('Center'),
			'right' => t('Right'),
		),
		'#required' => true,
	);

	$form['show_enrolled_in_courses'] = array(
		'#type' => 'checkbox',
		'#title' => t('Show courses you are enrolled in'),
		'#parents' => array('params', 'show_enrolled_in_courses'),
		'#default_value' => variable_get($key . 'show_enrolled_in_courses', 0),
	);

	$form['show_last_attended_courses'] = array(
		'#type' => 'checkbox',
		'#title' => t('Show last attended course'),
		'#parents' => array('params', 'show_last_attended_courses'),
		'#default_value' => variable_get($key . 'show_last_attended_courses', 0),
	);

	$form['introduction_2'] = array(
		'#markup' => '<h4>' . t('Customize your SSO box') . '</h4>',
	);

	$customCss = variable_get($key . 'custom_css', '');

	if ($customCss == '') {
		$form['edit_css'] = array(
			'#markup' => '<a href="javascript:void(0);" class="custom-css-toggle">' . t('Manually edit the CSS') . '</a>',
		);	
	}

	$form['custom_css'] = array(
		'#type' => 'textarea',
		'#title' => t('Custom CSS'),
		'#resizable' => false,
		'#rows' => 5,
		'#attributes' => array(
			'class' => array('custom-css'),
		),
		'#parents' => array('params', 'custom_css'),
		'#default_value' => variable_get($key . 'custom_css', ''),
	);

	$form['text_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Text color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'text_color'),
		'#default_value' => variable_get($key . 'text_color', ''),
	);

	$form['title_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Title color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'title_color'),
		'#default_value' => variable_get($key . 'title_color', ''),
	);

	$form['border_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Border color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'border_color'),
		'#default_value' => variable_get($key . 'border_color', ''),
	);

	$form['background_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Background color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'background_color'),
		'#default_value' => variable_get($key . 'background_color', ''),
	);

	$form['use_box_shadow'] = array(
		'#type' => 'checkbox',
		'#title' => t('Use box shadow?'),
		'#parents' => array('params', 'use_box_shadow'),
		'#default_value' => variable_get($key . 'use_box_shadow', 0),
	);

	$form['btn_background_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button background color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_background_color'),
		'#default_value' => variable_get($key . 'btn_background_color', ''),
	);

	$form['btn_border_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button border color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_border_color'),
		'#default_value' => variable_get($key . 'btn_border_color', ''),
	);

	$form['btn_text_color'] = array(
		'#type' => 'textfield',
		'#description' => '<div class="docebo-colorpicker"></div>',
		'#title' => t('Button text color'),
		'#attributes' => array('class' => array('edit-docebo-colorpicker')),
		'#parents' => array('params', 'btn_text_color'),
		'#default_value' => variable_get($key . 'btn_text_color', ''),
	);

	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => 'Save',
	);

	return $form;
}

function docebo_lms_sso_settings_form_validate($form, &$form_state) {
	$values = $form_state['values']['params'];

	if (!empty($values['box_width'])) {
		if (!is_numeric($values['box_width'])) {
			form_set_error('params][box_width', 'Box width must be a numeric value');
		} elseif ($values['box_width'] <= 0) {
			form_set_error('params][box_width', 'Box width must be greater than zero');
		}
	}

	if (!empty($values['box_height'])) {
		if (!is_numeric($values['box_height'])) {
			form_set_error('params][box_height', 'Box height must be a numeric value');
		} elseif ($values['box_height'] <= 0) {
			form_set_error('params][box_height', 'Box height must be greater than zero');
		}
	}
}

function docebo_lms_sso_settings_form_submit($form, &$form_state) {
	foreach ($form_state['values']['params'] as $key => $value) {
		variable_set('docebo_lms_sso_' . $key, $value);
	}

	$settings = docebo_lms_load_settings('docebo_lms_sso_');

	docebo_lms_generate_css('docebo.sso', $settings);
	docebo_lms_generate_custom_css('docebo.sso.custom', $settings);
	_drupal_flush_css_js();

	drupal_set_message(t('Configuration options have been saved'));
}

function docebo_lms_sync_form($form, &$form_state) {
	$scenario = variable_get('docebo_lms_scenario', 'login_mask');

	if ($scenario == 'login_mask') {
		drupal_set_message(t('User sync is not enabled. Please, see the !url.', array('!url' => l(t('General Settings'), 'admin/config/services/docebo'))), 'error');
		return array();
	}

	$query = db_select('users', 'u')->condition('uid', 0, '>');
	$query->addExpression('COUNT(u.uid)');
	$users['drupal'] = $query->execute()->fetchField();

	$api = new DoceboApi;
	$users['docebo'] = $api->getUsersCount();

	$form = array();
	$form['info'] = array(
		'#markup' => theme('docebo_sync_info', array('count' => $users)),
	);

	$form['sync'] = array(
		'#type' => 'submit',
		'#value' => t('Sync users'),
	);

	return $form;
}

function docebo_lms_sync_form_submit($form, &$form_state) {
	$api = new DoceboApi;

	$query = db_select('users', 'u');
	$query->condition('uid', 0, '>')
				->fields('u', array('uid', 'name', 'mail'));
	$result = $query->execute();

	$users = array();
	foreach ($result as $row) {
		$users[] = array(
			'ext_user' => $row->uid,
			'ext_user_type' => 'drupal',
			'wp_user_id' => $row->uid,
			'wp_username' => $row->name,
			'userid' => $row->name,
			'email' => $row->mail,
			'first_name' => '',
			'last_name' => '',
		);
	}

	if (!empty($users)) {
		$result = $api->syncUsers($users);

		if ($result !== false) {
			if ($result == 0) {
				drupal_set_message(t('No new users have been synced'));
			} else {
				drupal_set_message(t('@users been synced', array('@users' => format_plural($result, '1 new user has', '@count new users have'))));
			}
		} else {
			drupal_set_message(t('Error occured while syncing users'), 'error');
		}	
	}
}

function docebo_lms_courses_form($form, &$form_state) {
	$form = array();
	$key = 'docebo_lms_courses_';

	$form['block_width'] = array(
		'#type' => 'textfield',
		'#title' => t('Course block width'),
		'#description' => t('Min Value: 170px - Max Value: 230px'),
		'#field_suffix' => t('px'),
		'#parents' => array('params', 'block_width'),
		'#default_value' => variable_get($key . 'block_width', ''),
		'#required' => true,
	);

	$form['display_count'] = array(
		'#type' => 'textfield',
		'#title' => t('Number of courses displayed'),
		'#description' => t('Leave blank if you want to display every course.'),
		'#parents' => array('params', 'display_count'),
		'#default_value' => variable_get($key . 'display_count', ''),
	);

	$form['submit'] = array(
		'#type' => 'submit',
		'#value' => t('Save changes'),
	);

	$form['preview'] = array(
		'#markup' => '<h4>' . t('Preview') . '</h4>' . theme('docebo_lms_courses'),
	);

	return $form;
}

function docebo_lms_courses_form_validate($form, &$form_state) {
	$values = $form_state['values']['params'];

	if (!is_numeric($values['block_width'])) {
		form_set_error('params][block_width', t('Block width must be numeric'));
	} elseif ($values['block_width'] < 170 || $values['block_width'] > 230) {
		form_set_error('params][block_width', t('Block width must be in range of 170..230 px'));
	}

	if (!empty($values['display_count'])) {
		if (!is_numeric($values['display_count'])) {
			form_set_error('params][display_count', t('Number of courses should be numeric'));
		} elseif ($values['display_count'] < 0) {
			form_set_error('params][display_count', t('Number of courses should equal or be greater than zero.'));
		}	
	}
}

function docebo_lms_courses_form_submit($form, &$form_state) {
	foreach ($form_state['values']['params'] as $key => $value) {
		variable_set('docebo_lms_courses_' . $key, $value);
	}
	drupal_set_message(t('Configuration options have been saved'));
}

function docebo_lms_admin_support_page() {
	global $base_url;

	$path = drupal_get_path('module', 'docebo_lms') . '/docebo_lms.info';
	$info = drupal_parse_info_file($path);

	$data = array(
		'site_url' => $base_url,
		'drupal_version' => VERSION,
		'module_version' => docebo_lms_parse_version($info['version']),
	);

	return theme('docebo_lms_support_page', $data);
}

function theme_docebo_lms_support_page($data) {
	$output = '<p>' . t('Please find the most frequently asked questions below. If these don\'t solve your problem, you may contact Docebo Support') . '</p>';
	$output .= '<h3>' . t('Support info') . '</h3>';
	$output .= '<p>' . t('If you copy this block of information into your ticket, it helps us answer your questions easier and quicker. Thank you!') . '</p>';
	$output .= '<p><strong>' . t('Website URL') . '</strong>: ' . $data['site_url'] . '</p>';
	$output .= '<p><strong>' . t('Drupal version') . '</strong>: ' . $data['drupal_version'] . '</p>';
	$output .= '<p><strong>' . t('Docebo module version') . '</strong>: ' . $data['module_version'] . '</p>';

	return $output;
}